@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Nicola Di Lieto <nicola.dilieto@gmail.com>
+ * Copyright (C) 2019,2020 Nicola Di Lieto <nicola.dilieto@gmail.com>
  *
  * This file is part of uacme.
  *
@@ -75,7 +75,7 @@ char *jws_thumbprint(privkey_t);
 char *jws_encode(const char *, const char *, privkey_t);
 keytype_t key_type(privkey_t);
 privkey_t key_load(keytype_t, int bits, const char *, ...);
-char *csr_gen(const char * const *, privkey_t);
+char *csr_gen(const char * const *, bool, privkey_t);
 char *cert_der_base64url(const char *);
 bool cert_valid(const char *, const char * const *, int);
 
