@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Nicola Di Lieto <nicola.dilieto@gmail.com>
+ * Copyright (C) 2019,2020 Nicola Di Lieto <nicola.dilieto@gmail.com>
  *
  * This file is part of uacme.
  *
@@ -72,14 +72,14 @@ typedef struct acme
 char *find_header(const char *headers, const char *name)
 {
     char *regex = NULL;
-    if (asprintf(&regex, "^%s: (.*)\r\n", name) < 0)
+    if (asprintf(&regex, "^%s:[ \t]*(.*)\r\n", name) < 0)
     {
         warnx("find_header: asprintf failed");
         return NULL;
     }
     char *ret = NULL;
     regex_t reg;
-    if (regcomp(&reg, regex, REG_EXTENDED | REG_NEWLINE))
+    if (regcomp(&reg, regex, REG_EXTENDED | REG_ICASE | REG_NEWLINE))
     {
         warnx("find_header: regcomp failed");
     }
@@ -582,10 +582,10 @@ bool account_retrieve(acme_t *a)
             acme_error(a);
             return false;
     }
-    if (json_compare_string(a->json, "status", "valid"))
+    const char* status = json_find_string(a->json, "status");
+    if (status && strcmp(status, "valid"))
     {
-        const char* status = json_find_string(a->json, "status");
-        warnx("invalid account status (%s)", status ? status : "unknown");
+        warnx("invalid account status (%s)", status);
         return false;
     }
     if (!(a->kid = find_header(a->headers, "Location")))
@@ -893,8 +893,10 @@ bool authorize(acme_t *a)
         bool chlg_done = false;
         for (size_t j=0; j<chlgs->v.array.size && !chlg_done; j++)
         {
-            if (json_compare_string(chlgs->v.array.values+j,
-                        "status", "pending") == 0)
+            const char *status = json_find_string(
+                    chlgs->v.array.values+j, "status");
+            if (status && (strcmp(status, "pending") == 0
+                        || strcmp(status, "processing") == 0))
             {
                 const char *url = json_find_string(
                         chlgs->v.array.values+j, "url");
@@ -1021,7 +1023,7 @@ out:
     return success;
 }
 
-bool cert_issue(acme_t *a)
+bool cert_issue(acme_t *a, bool status_req)
 {
     bool success = false;
     char *csr = NULL;
@@ -1109,7 +1111,7 @@ bool cert_issue(acme_t *a)
     }
 
     msg(1, "generating certificate request");
-    csr = csr_gen(a->names, a->dkey);
+    csr = csr_gen(a->names, status_req, a->dkey);
     if (!csr)
     {
         warnx("failed to generate certificate signing request");
@@ -1371,10 +1373,11 @@ void usage(const char *progname)
 {
     fprintf(stderr,
         "usage: %s [-a|--acme-url URL] [-b|--bits BITS] [-c|--confdir DIR]\n"
-        "\t[-d|--days DAYS] [-f|--force] [-h|--hook PROGRAM] [-n|--never-create]\n"
-        "\t[-s|--staging] [-t|--type RSA | EC] [-v|--verbose ...] [-V|--version]\n"
-        "\t[-y|--yes] [-?|--help] new [EMAIL] | update [EMAIL] | deactivate\n"
-        "\t| newkey | issue DOMAIN [ALTNAME ...]] | revoke CERTFILE\n", progname);
+        "\t[-d|--days DAYS] [-f|--force] [-h|--hook PROGRAM] [-m|--must-staple]\n"
+        "\t[-n|--never-create] [-s|--staging] [-t|--type RSA | EC]\n"
+        "\t[-v|--verbose ...] [-V|--version] [-y|--yes] [-?|--help]\n"
+        "\tnew [EMAIL] | update [EMAIL] | deactivate | newkey |\n"
+        "\tissue DOMAIN [ALTNAME ...]] | revoke CERTFILE\n", progname);
 }
 
 int main(int argc, char **argv)
@@ -1388,6 +1391,7 @@ int main(int argc, char **argv)
         {"force",        no_argument,       NULL, 'f'},
         {"help",         no_argument,       NULL, '?'},
         {"hook",         required_argument, NULL, 'h'},
+        {"must-staple",  no_argument,       NULL, 'm'},
         {"never-create", no_argument,       NULL, 'n'},
         {"staging",      no_argument,       NULL, 's'},
         {"type",         required_argument, NULL, 't'},
@@ -1404,6 +1408,7 @@ int main(int argc, char **argv)
     bool yes = false;
     bool staging = false;
     bool custom_directory = false;
+    bool status_req = false;
     int days = 30;
     int bits = 0;
     keytype_t type = PK_RSA;
@@ -1446,7 +1451,7 @@ int main(int argc, char **argv)
     {
         char *endptr;
         int option_index;
-        int c = getopt_long(argc, argv, "a:b:c:d:f?h:nst:vVy",
+        int c = getopt_long(argc, argv, "a:b:c:d:f?h:mnst:vVy",
                 options, &option_index);
         if (c == -1) break;
         switch (c)
@@ -1491,6 +1496,10 @@ int main(int argc, char **argv)
                 a.hook = optarg;
                 break;
 
+            case 'm':
+                status_req = true;
+                break;
+
             case 'n':
                 never = true;
                 break;
@@ -1779,7 +1788,8 @@ int main(int argc, char **argv)
             }
         }
 
-        if (acme_bootstrap(&a) && account_retrieve(&a) && cert_issue(&a))
+        if (acme_bootstrap(&a) && account_retrieve(&a)
+                && cert_issue(&a, status_req))
         {
             ret = 0;
         }
